@@ -7,6 +7,7 @@
 #' @param X2 second \code{matrix} to be compared (\code{data.frames} are also accepted).
 #' @param ncomp1 (GCD) number of subspace components from the first \code{matrix} (default: full subspace).
 #' @param ncomp2 (GCD) number of subspace components from the second \code{matrix} (default: full subspace).
+#' @param center \code{logical} indicating if input matrices should be centered (default = TRUE).
 #'
 #' @details Details can be found in Ramsey's paper:
 #' \itemize{
@@ -14,7 +15,7 @@
 #'  \item{r2:}{ orientation-independent inner product correlation}
 #'  \item{r3:}{ spectra-independent inner product correlations (including orientation)}
 #'  \item{r4:}{ Spectra-Independent inner product Correlations}
-#'  \item{GCD:}{ Yanai's GCD Measure. To reproduce the original GCD, use all components.}
+#'  \item{GCD:}{ Yanai's Generalized Coefficient of Determination (GCD) Measure. To reproduce the original GCD, use all components. When \code{X1} and \code{X2} are dummy variables, GCD is proportional with Pillai's criterion: tr(W^-1(B+W)).}
 #' }
 #'
 #' @return A single value measuring the similarity of two matrices.
@@ -38,13 +39,25 @@
 #' GCD(X1,X2, 5,5)
 #'
 #' @export
-r1 <- function(X1, X2){
+r1 <- function(X1, X2, center = TRUE){
+  X1 <- as.matrix(X1)
+  X2 <- as.matrix(X2)
+  if(center){
+    X1 <- X1 - rep(colMeans(X1), each = nrow(X1))
+    X2 <- X2 - rep(colMeans(X2), each = nrow(X1))
+  }
   Trace(crossprod(X1,X2))/sqrt(Trace(crossprod(X1))*Trace(crossprod(X2)))
 }
 
 #' @rdname r1
 #' @export
-r2 <- function(X1, X2){
+r2 <- function(X1, X2, center = TRUE){
+  X1 <- as.matrix(X1)
+  X2 <- as.matrix(X2)
+  if(center){
+    X1 <- X1 - rep(colMeans(X1), each = nrow(X1))
+    X2 <- X2 - rep(colMeans(X2), each = nrow(X1))
+  }
   udv_x1 <- svd(X1)
   udv_x2 <- svd(X2)
   r1(udv_x1$u %*% diag(udv_x1$d),udv_x2$u %*% diag(udv_x2$d))
@@ -52,7 +65,13 @@ r2 <- function(X1, X2){
 
 #' @rdname r1
 #' @export
-r3 <- function(X1, X2){
+r3 <- function(X1, X2, center = TRUE){
+  X1 <- as.matrix(X1)
+  X2 <- as.matrix(X2)
+  if(center){
+    X1 <- X1 - rep(colMeans(X1), each = nrow(X1))
+    X2 <- X2 - rep(colMeans(X2), each = nrow(X1))
+  }
   udv_x1 <- svd(X1)
   udv_x2 <- svd(X2)
   r1(tcrossprod(udv_x1$u, udv_x1$v),tcrossprod(udv_x2$u, udv_x2$v))
@@ -60,7 +79,13 @@ r3 <- function(X1, X2){
 
 #' @rdname r1
 #' @export
-r4 <- function(X1, X2){
+r4 <- function(X1, X2, center = TRUE){
+  X1 <- as.matrix(X1)
+  X2 <- as.matrix(X2)
+  if(center){
+    X1 <- X1 - rep(colMeans(X1), each = nrow(X1))
+    X2 <- X2 - rep(colMeans(X2), each = nrow(X1))
+  }
   udv_x1 <- svd(X1)
   udv_x2 <- svd(X2)
   r1(udv_x1$u, udv_x2$u)
@@ -68,8 +93,15 @@ r4 <- function(X1, X2){
 
 #' @rdname r1
 #' @export
-GCD <- function(X1, X2, ncomp1 = Rank(X1)-1, ncomp2 = Rank(X2)-1){
+GCD <- function(X1, X2, ncomp1 = Rank(X1), ncomp2 = Rank(X2), center = TRUE){
+  X1 <- as.matrix(X1)
+  X2 <- as.matrix(X2)
+  if(center){
+    X1 <- X1 - rep(colMeans(X1), each = nrow(X1))
+    X2 <- X2 - rep(colMeans(X2), each = nrow(X1))
+  }
   udv_x1 <- svd(X1)
   udv_x2 <- svd(X2)
-  r1(tcrossprod(udv_x1$u[,1:ncomp1]), tcrossprod(udv_x2$u[,1:ncomp2]))
+  r1(tcrossprod(udv_x1$u[,1:ncomp1,drop=FALSE]),
+     tcrossprod(udv_x2$u[,1:ncomp2,drop=FALSE]))
 }
